﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("Stochastic MultiComponentVector Triple Move Generator", "Generates all possible multi component moves from a given multiComponentVector.")]
  [StorableClass]
  public class StochasticMCVTripleMoveGenerator : MultiComponentVectorMoveGenerator, IMultiMoveGenerator, IStochasticOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    public IValueLookupParameter<IntValue> SampleSizeParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["SampleSize"]; }
    }

    public IntValue SampleSize {
      get { return SampleSizeParameter.Value; }
      set { SampleSizeParameter.Value = value; }
    }


    [StorableConstructor]
    protected StochasticMCVTripleMoveGenerator(bool deserializing) : base(deserializing) { }
    protected StochasticMCVTripleMoveGenerator(StochasticMCVTripleMoveGenerator original, Cloner cloner) : base(original, cloner) { }
    public StochasticMCVTripleMoveGenerator()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator."));
      Parameters.Add(new ValueLookupParameter<IntValue>("SampleSize", "The number of moves to generate."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new StochasticMCVTripleMoveGenerator(this, cloner);
    }

    public static List<MultiComponentVectorMove> GenerateMCVTripleMoves(MultiComponentVectorEncoding mcv, IRandom random, int sampleSize) {
      var result = new List<MultiComponentVectorMove>();
      var moves = new List<MultiComponentVectorMove> (ExhaustiveMCVTripleMoveGenerator.GenerateMCVTripleMoves(mcv));
      int moveCount = moves.Count;
      for (int i = 0; i < sampleSize; i++) {
        result.Add(moves [random.Next(moveCount)]);
      }
      return result;
    }

    public static MultiComponentVectorMove[] Apply(MultiComponentVectorEncoding multiComponentVector, IRandom random, int sampleSize) {
      var generatedMoves = GenerateMCVTripleMoves(multiComponentVector, random, sampleSize);
      int totalMoves = generatedMoves.Count;
      MultiComponentVectorMove[] moves = new MultiComponentVectorMove[totalMoves];
      int count = 0;
      foreach (MultiComponentVectorMove move in generatedMoves) {
        moves[count++] = move;
      }
      return moves;
    }

    protected override MultiComponentVectorMove[] GenerateMoves(MultiComponentVectorEncoding multiComponentVector) {
      IRandom random = RandomParameter.ActualValue;
      return Apply(multiComponentVector, random, SampleSizeParameter.ActualValue.Value);
    }
  }
}
